(** Arbres de syntaxe abstraite pour oPCF *)

type label = string

(** Type de la syntaxe abstraite des termes de iPCF. *)
type t =
  | Num of int               (** Constante entire *)
  | Var of string            (** Accs  une variable *)
  | Op  of Op.t * t * t        (** Application d'un oprateur [t1 op t2] *)
  | Ifz of t * t * t         (** Conditionnelle [Ifz t1 Then t2 Else t3] *)
  | Let of string * t * t    (** Liaison locale [Let x = t1 In t2] *) 
  | App of t * t             (** Application de fonction [t1 t2] *)
  | Fun of string * t        (** Dfinition de fonction [Fun x -> t] *)
  | Fix of string * t        (** Point fixe [Fix x -> t] *)
  | Rec      of (label * t) list      (** Cration d'un enregistrement *)
  | Access   of t * label             (** Accs  un champ d'enregistrement *)
  | CopyWith of t * (label * t) list  (** Copie/extension d'enregistrement *)
(** Objets *)
  | Obj of string option *  (label * t) list
  | Invoke of t * label
  | ObjWith of string option  * t * (label * t) list

type 'a env = (string * 'a) list
