(** PCF explicitement typ *)

(** Types des types *)
module Type : sig

  type t =
    | Nat        (** Type des entiers naturels [Nat] *)
    | Arrow of t * t (** Type des fonctions [A -> B] *)
    | Tvar of int (** Variable de type (TD 6) *)

  (** Afficher un type dans un canal de sortie. *)
  val print : out_channel -> t -> unit

  (** Reprsenter un type dans une chane *)
  val to_string : t -> string
end


(** Les termes de la syntaxe abstraite type *)
module Ast : sig


(** Les termes *)
type t =
  | Num of int              (** Constante entire *)
  | Var of string           (** Accs  une variable *)
  | Op  of Op.t * t * t    (** Application d'un oprateur binaire [t1 op t2] *)
  | Ifz of t * t * t        (** Conditionnelle [Ifz t1 Then t2 Else t3] *)
  | Let of string * t * t   (** Liaison locale [Let x = t1 In t2] *) 
  | App of t * t            (** Application de fonction [t1 t2]*)
  | Fun of string * Type.t * t  (** Dfinition de fonction [Fun (x:ty) -> t] *)
  | Fix of string * Type.t * t  (** Point fixe [Fix (x:ty) -> t] *)

(** Efface les types d'un terme de PCF explicitement typ, ce qui
    produit un terme de PCF non typ *)
val erase : t -> S.Ast.t

end

(** Lecteur des termes de PCF typ, comme {!S.Reader} *)
module Reader : sig

  val from_chan : in_channel -> Ast.t
(** Dans un canal *)


  val from_string : string -> Ast.t
(** Dans une chane *)

end

(** Boucle interactive pour PCF typ, voir aussi {!S.Top} *)
module Top : sig

  val loop :  (Type.t Env.t -> Ast.t -> Type.t) -> unit
(** Boucle interactive pour PCF typ, limite aux types.

    L'appel [loop check] appelle indfiniment le vrificateur [check],
    avec un environnement tendu par types des dfinitions. *)

  val loop_eval : (Type.t Env.t -> Ast.t -> Type.t) ->
     (out_channel -> 'a -> unit) ->
     ('a Env.t -> Ast.t -> 'a) -> unit
(** L'appel [loop_eval check print eval] lance une boucle interactive
     complte qui combine vrification de type
    (par [check]) et valuation (par [eval]).
    Les valeurs (type ['a]) sont affiches par [print].

    Un double environnement (pour les types {!Type.t} et les valeurs ['a])
    est maintenu. *)
end

