
type t =
  | Num of int              (** Constante entire *)
  | Var of string           (** Accs  une variable *)
  | Op  of Op.t * t * t    (** Application d'un oprateur binaire [t1 op t2] *)
  | Ifz of t * t * t        (** Conditionnelle [Ifz t1 Then t2 Else t3] *)
  | Let of string * t * t   (** Liaison locale [Let x = t1 In t2] *) 
  | App of t * t            (** Application de fonction [t1 t2]*)
  | Fun of string * Type.t * t  (** Dfinition de fonction [Fun (x:ty) -> t] *)
  | Fix of string * Type.t * t  (** Point fixe [Fix (x:ty) -> t] *)

let rec erase e = match e with
| Num i -> S.Ast.Num i
| Var x -> S.Ast.Var x
| Op (op,e1,e2) ->
    S.Ast.Op (op, erase e1, erase e2)
| Ifz (e1,e2,e3) ->
    S.Ast.Ifz (erase e1, erase e2, erase e3)
| Let (x,e1,e2) ->
    S.Ast.Let (x,erase e1, erase e2)
| App (e1,e2) ->
    S.Ast.App (erase e1, erase e2)
| Fun (x,_,e) ->
    S.Ast.Fun (x,erase e)
| Fix (x,t,e) ->
    S.Ast.Fix (x,erase e)
